<?xml version="1.0" encoding="UTF-8"?>
<xsl:stylesheet version="2.0"
	xmlns="http://www.w3.org/1999/xhtml"
	xmlns:h="http://www.w3.org/1999/xhtml"
	xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
	xmlns:fn="http://www.w3.org/2005/xpath-functions"
	xmlns:xs="http://www.w3.org/2001/XMLSchema"
	xmlns:o="https://trac.frantovo.cz/odstavcovac-TODO-XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX-/wiki/xmlns/odstavcovac"
	exclude-result-prefixes="fn h xs o">
	<xsl:output method="xml" indent="yes" encoding="UTF-8" omit-xml-declaration="yes"/>
	
	<xsl:param name="cssTřída"/>
	
	<!-- Celý dokument -->
	<xsl:template match="/">
		<html>
			<body>
			
				<xsl:variable name="prvníKolo">
					<xsl:apply-templates select="h:html/h:body/node()" mode="prvníKolo"/>
				</xsl:variable>
				
				<xsl:variable name="druhéKolo">
					<xsl:apply-templates select="$prvníKolo" mode="druhéKolo"/>
				</xsl:variable>
				
				<xsl:apply-templates select="$druhéKolo" mode="třetíKolo"/>
				
			</body>
		</html>
	</xsl:template>
	
	
	<!-- Kopírujeme elementy, ale vynecháme nepoužité xmlns deklarace: -->
	<xsl:template match="*" mode="kopíruj">
		<xsl:element name="{name()}">
			<xsl:copy-of select="@*"/>
			<xsl:apply-templates mode="kopíruj"/>
		</xsl:element>
    </xsl:template>
    
    
    <!-- Mezi odstavci je prázdný řádek, můžou být mezery/tabulátory. -->
	<xsl:variable name="oddělovač" select="'\n\s*\n\s*'"/>
    
	
	<!-- Funkce: zda jde o XHTML inline element – může se vyskytovat uvnitř odstavců. -->
	<xsl:template name="inlineElement" as="xs:boolean">
		<xsl:param name="prvek"/>
		<xsl:sequence select="
			$prvek/name() = 'a' or						
			$prvek/name() = 'abbr' or						
			$prvek/name() = 'acronym' or						
			$prvek/name() = 'b' or						
			$prvek/name() = 'br' or						
			$prvek/name() = 'cite' or						
			$prvek/name() = 'code' or						
			$prvek/name() = 'em' or						
			$prvek/name() = 'i' or						
			$prvek/name() = 'img' or						
			$prvek/name() = 'q' or
			$prvek/name() = 'span' or
			$prvek/name() = 'strong' or
			$prvek/name() = 'sub' or
			$prvek/name() = 'sup' or
			$prvek/name() = 'tt' or
			$prvek/name() = 'u' or
			$prvek/name() = 'var'
			"/>
		<!-- …případně další, pokud je budeme chtít podporovat. -->
	</xsl:template>
	
	
	<!-- Funkce: zda je prvek začátkem odstavce. -->
	<xsl:template name="začátekOdstavce" as="xs:boolean">
		<xsl:param name="prvek"/>
		
		<xsl:variable name="inlineElement" as="xs:boolean">
			<xsl:call-template name="inlineElement"><xsl:with-param name="prvek" select="$prvek"/></xsl:call-template>
		</xsl:variable>
		
		<xsl:variable name="předchůdce" select="$prvek/preceding-sibling::node()[1]"/>
		
		<xsl:variable name="inlineElementPředchůdce" as="xs:boolean">
			<xsl:call-template name="inlineElement"><xsl:with-param name="prvek" select="$předchůdce"/></xsl:call-template>
		</xsl:variable>
		
		<xsl:variable name="textovýUzel" select="boolean($prvek/self::text())"/>
				
		<xsl:sequence select="
			($inlineElement or $textovýUzel) 
			and 
			(
				($inlineElementPředchůdce and matches($prvek, concat('^', $oddělovač, '.*')))
				or
				not($inlineElementPředchůdce)
				or
				not($předchůdce)
			)
			and
			(
				not($předchůdce/self::text())
				or
				matches($předchůdce/self::text(), concat('.*', $oddělovač, '$'))
			)
			"/>
	</xsl:template>
	
	
	<!--
		V prvním kole zavřeme volný text a inline elementy do značek <o:odstavec typ=""/>,
		kde typ může být "začátek", což značí, že se jedná o první část budoucího odstavce <p/>.
	-->
	<xsl:template match="text()" mode="prvníKolo">
	
		<xsl:variable name="začátekOdstavce" as="xs:boolean">
			<xsl:call-template name="začátekOdstavce">
				<xsl:with-param name="prvek" select="."/>
			</xsl:call-template>
		</xsl:variable>
		
		<xsl:for-each select="fn:tokenize(., $oddělovač)">
			<xsl:element name="o:odstavec">
				<xsl:if test="$začátekOdstavce or not(position() = 1)">
					<xsl:attribute name="typ">začátek</xsl:attribute>
				</xsl:if>
				<xsl:value-of select="."/>
			</xsl:element>
		</xsl:for-each>
	
	</xsl:template>
	
	<!-- 
		Inline elementy zavíráme do <o:odstavec typ=""/>,
		ostatní vkládáme, jak jsou.
	-->
	<xsl:template match="*" mode="prvníKolo">
	
		<xsl:variable name="inlineElement" as="xs:boolean">
			<xsl:call-template name="inlineElement">
				<xsl:with-param name="prvek" select="."/>
			</xsl:call-template>
		</xsl:variable>
		
		<xsl:choose>
			<!-- TODO: zvláštní šablona (match="…") pro inline elementy místo větvení? -->
			<xsl:when test="$inlineElement">
				<xsl:variable name="začátekOdstavce" as="xs:boolean">
					<xsl:call-template name="začátekOdstavce">
						<xsl:with-param name="prvek" select="."/>
					</xsl:call-template>
				</xsl:variable>
				<xsl:element name="o:odstavec">
					<xsl:if test="$začátekOdstavce">
						<xsl:attribute name="typ">začátek</xsl:attribute>
					</xsl:if>
					<xsl:copy-of select="."/>
				</xsl:element>
			</xsl:when>
			<xsl:otherwise>				
				<xsl:copy-of select="."/>
			</xsl:otherwise>		
		</xsl:choose>
	
	</xsl:template>
	
	<!-- V druhém kole spojíme jednotlivé části odstavců. -->
	<xsl:template match="o:odstavec[@typ='začátek']" mode="druhéKolo">
		<o:odstavec>
			<xsl:call-template name="spojOdstavce">
				<xsl:with-param name="část" select="."/>
			</xsl:call-template>
		</o:odstavec>
	</xsl:template>
	<!-- Následující části odstavce přeskočíme – postará se o ně vnitřní smyčka volaná z předchozí šablony. -->
	<xsl:template match="o:odstavec" mode="druhéKolo"/>
	<!-- Neinline (blokové) elementy vložíme, jak jsou. -->
	<xsl:template match="*" mode="druhéKolo">
		<xsl:copy-of select="."/>
	</xsl:template>
	
	
	<!--
		Za první část (parametr, <o:odstavec typ="začátek"/>) resp. její vnitřek
		připojíme (rekurze) všechny další části téhož odstavce (oddělíme mezerou).
		Konec odstavce poznáme tak, že následovník je něco jiného než <o:odstavec/> nebo má atribut typ="začátek".
	-->
	<xsl:template name="spojOdstavce">
		<xsl:param name="část"/>
		<xsl:copy-of select="$část/child::node()"/>
		<xsl:variable name="následovník" select="$část/following-sibling::node()[1]"/>
		<xsl:if test="$následovník/name() = 'o:odstavec' and not($následovník/@typ = 'začátek')">
			<xsl:text> </xsl:text>
			<xsl:call-template name="spojOdstavce">
				<xsl:with-param name="část" select="$následovník"/>
			</xsl:call-template>
		</xsl:if>
	</xsl:template>
	
	
	<!-- Ve třetím kole smažeme prázdné mešuge odstavce. -->
	<xsl:template mode="třetíKolo" match="o:odstavec[
		count(child::node()) = 0 
		or 
		(
			count(child::node()) = 1 
			and
			text()
			and
			matches(text(), '^\s*$')
		)
		]">
		<xsl:text> </xsl:text>	
	</xsl:template>
	<!-- Převedeme z <o:odstavec/> na <p/> -->
	<xsl:template match="o:odstavec" mode="třetíKolo">
		<xsl:element name="p">
			<xsl:if test="$cssTřída">
				<xsl:attribute name="class"><xsl:value-of select="$cssTřída"/></xsl:attribute>
			</xsl:if>
			<xsl:apply-templates select="child::node()" mode="kopíruj"/>
		</xsl:element>
	</xsl:template>
	<!-- Všechno ostatní zkopírujeme, jak je. -->
	<xsl:template match="*" mode="třetíKolo">
		<xsl:copy-of select="."/>
	</xsl:template>
	
	
</xsl:stylesheet>
