<?xml version="1.0" encoding="UTF-8"?>
<xsl:stylesheet version="2.0"
	xmlns="http://www.w3.org/1999/xhtml"
	xmlns:h="http://www.w3.org/1999/xhtml"
	xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
	xmlns:fn="http://www.w3.org/2005/xpath-functions"
	xmlns:svg="http://www.w3.org/2000/svg"
	xmlns:xs="http://www.w3.org/2001/XMLSchema"
	exclude-result-prefixes="fn h xs">
	<xsl:output 
		method="xml" 
		indent="yes" 
		encoding="UTF-8"		
		doctype-public="-//W3C//DTD XHTML 1.1 plus MathML 2.0 plus SVG 1.1//EN"/><!-- 
		doctype-system="http://www.w3.org/2002/04/xhtml-math-svg/xhtml-math-svg.dtd"/>-->
	
		
	<xsl:param name="title"/>
	<xsl:param name="headComment"/>
	<xsl:param name="isRoot"/>
	<xsl:param name="urlBase"/>
	<xsl:param name="wwwRead"/>
	<xsl:param name="wwwPost"/>
	
	
	<!-- Celý dokument -->
	<xsl:template match="/">
		<html>
			<head>
				<meta http-equiv="Content-Type" content="application/xhtml+xml; charset=utf-8" />
				<xsl:if test="$urlBase">
					<base href="{$urlBase}"/>
				</xsl:if>
				<xsl:if test="$title">
					<title><xsl:value-of select="$title"/></title>
				</xsl:if>
				<xsl:if test="$headComment">
					<xsl:comment><xsl:value-of select="$headComment"/></xsl:comment>
				</xsl:if>
				<style type="text/css">
					body {
						font-family: sans-serif;
						font-size: 16px;
					}
					
					.wwwLinks {
						border-top: 2px solid grey;
						margin-top: 16px;
						font-size: 66%;
					}
					
					blockquote {
						background-color: #eee;
						font-style: italic;
						padding: 2px 20px;
						margin: 10px 32px;
						border-left: 3px solid silver;
					}
					
					pre {
						background-color: #eee;
						padding: 2px 20px;
						margin: 10px 32px;
						border-left: 3px solid #a00;
					}
					
					img {
						margin: 8px;
					}
					
					a img {
						border: none;
					}
				</style>
			</head>
			<body>
				<xsl:if test="$title and $isRoot">
					<h1><xsl:value-of select="$title"/></h1>			
				</xsl:if>
								
				<xsl:apply-templates select="h:html/h:body"/>
				
				<xsl:if test="$wwwRead or $wwwPost">
					<div class="wwwLinks">
						<p>Přístup přes síť www:</p>
						<ul>
							<xsl:if test="$wwwRead"><li><a href="{$wwwRead}">číst tento příspěvek</a></li></xsl:if>
							<xsl:if test="$wwwPost"><li><a href="{$wwwPost}">odpovědět na tento příspěvek</a></li></xsl:if>
						</ul>
					</div>
				</xsl:if>
			</body>
		</html>
	</xsl:template>
	
	
	<!-- Odkazy -->
	<xsl:template match="h:a">
		<a href="{@href}" title="{@title}"><xsl:apply-templates/></a>
	</xsl:template>
	
	
	<!-- Obrázky -->
	<xsl:template match="h:img">
		<img src="{@src}" alt="{@alt}" title="{@title}"><xsl:apply-templates/></img>
	</xsl:template>
	
	<!-- Zkratky -->
	<xsl:template match="h:abbr">
		<abbr title="{@title}"><xsl:apply-templates/></abbr>
	</xsl:template>
	
	<xsl:template match="h:blockquote[not(h:p)]">
		<blockquote>
			<p>
				<xsl:copy-of select="node()"/>
			</p>
		</blockquote>
	</xsl:template>
	
	
	<!-- Další povolené značky – ostatní odfiltrujeme (zbude z nich jen text) -->
	<xsl:template match="*">
		<xsl:choose>		
			<xsl:when test="
				name() = 'h1' or
				name() = 'h2' or
				name() = 'h3' or
				name() = 'h4' or
				name() = 'h5' or
				name() = 'h6' or
				name() = 'p' or
				name() = 'div' or
				name() = 'br' or
				name() = 'hr' or
				name() = 'strong' or
				name() = 'em' or
				name() = 'sub' or
				name() = 'sup' or
				name() = 'del' or
				name() = 'ul' or
				name() = 'ol' or
				name() = 'li' or
				name() = 'pre' or
				name() = 'code' or
				name() = 'blockquote'">
				<xsl:element name="{name()}">
					<xsl:copy-of select="./node()"/>
				</xsl:element>
			</xsl:when>
			<xsl:otherwise>
				<xsl:apply-templates/>
			</xsl:otherwise>
		</xsl:choose>		
    </xsl:template>
	
</xsl:stylesheet>
